#include "myopenglrenderingwindow.h"
#include "QDebug"
///////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////
myOpenGLRendererWindow::myOpenGLRendererWindow(QOpenGLWidget *parent) : QOpenGLWidget(parent)
{
    QSurfaceFormat format;
    format.setDepthBufferSize(24);
    format.setVersion(2,0);
    format.setProfile(QSurfaceFormat::CoreProfile);
    QSurfaceFormat::setDefaultFormat( format);

    OGL_Context = new QOpenGLContext();
    OGL_Context->setFormat(format);
    OGL_Context->create();
    if(OGL_Context->isValid())
    {
        qDebug( " OGL_Context is Valid " );
        if (OGL_Context->openGLModuleType()==QOpenGLContext::LibGL)
        {
            qDebug( " OGL_Context is LibGL " );
        }
        else if(OGL_Context->openGLModuleType()==QOpenGLContext::LibGLES)
        {
            qDebug( " OGL_Context is LibGLES " );
        }
    }
    else
    {
        qDebug( " INVALID OGL_Context" );
    };
}
///////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////
void myOpenGLRendererWindow::initializeGL()
{
    qDebug() << QString(" initializeGL ");
    initializeOpenGLFunctions();
}
///////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////
void myOpenGLRendererWindow::paintGL()
{
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    SetViewPort();//tell gl what part of screen it should draw
    SetViewProjectionMatrix();//set my camera and projection
    DrawMyContent();//start drawing
}
///////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////
void myOpenGLRendererWindow::resizeGL(int width, int height)
{
    qDebug() << QString(" resizeGL : New width ") << width << QString(" New height ") << height ;

    float aspect= (float)width / (float)height;

    //These will hold what values are passed during resize
    myUnproject_Width = width ;
    myUnproject_Height = height ;

    //dont uncomment these, they dont affect viewport
    //float nearPlane=1.0f;float FarPlane=1000.0f;float PerspectiveAngle=45.0f;
    //projectionMatrix.setToIdentity();
    //projectionMatrix.perspective(PerspectiveAngle, aspect, nearPlane, FarPlane);
    //SetViewProjectionMatrix();
}
///////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////
void myOpenGLRendererWindow::mouseMoveEvent(QMouseEvent *event)
{
    //check debug log
    //if u want to see constant output just uncomment both
    //qDebug() << QString(" Mouse Coordinates : x= ") << event->x() << " y= " <<  event->y() ;
    //MirrorMirrorTellMetheTruth();
    //update()
}
///////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////
void myOpenGLRendererWindow::mousePressEvent(QMouseEvent *event)
{
    //check debug log
    //qDebug() << QString(" Mouse Coordinates : x= ") << event->x() << " y= " <<  event->y() ;
    MirrorMirrorTellMetheTruth();
}
///////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////
void myOpenGLRendererWindow::MirrorMirrorTellMetheTruth()
{
    //Using openglfunction to retrieve the GL viewport , it should match the actual viewable are of the screen
    //where renderind/drawing happens
    int picking_viewport[4];
    glGetIntegerv( GL_VIEWPORT, picking_viewport );//this returns wrong
    //outputing the GL_VIEWPORT
    qDebug() << QString(" glGetIntegerv GL_VIEWPORT reports : width= ") << picking_viewport[2] << " height= " <<  picking_viewport[3] ;
    //outputing what the widget think is its dimensions (which in my opinion are the correct)
    qDebug() << QString(" QWidget reports : width= ") << QWidget::width() << " height= " <<  QWidget::height() ;

    //The GL_VIEWPORT are always greater than the Widget
    // GL_VIEWPORT is matching the centralwin dimensions
    //qDebug() << QString(" The opengl window's dimensions are reported correctly from the Qwidget ");
}
///////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////
void myOpenGLRendererWindow::SetViewPort()
{
    //Here i tell it to get the whole window of the widget

    //glViewport(0,0,width(),height());
    glViewport(0,0,QWidget::width(),QWidget::height());
}
///////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////
void myOpenGLRendererWindow::SetViewProjectionMatrix()
{
//dummy , i ll not show anything and even if i did,it would not change the viewport size
}
///////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////
void myOpenGLRendererWindow::DrawMyContent()
{
//dummy , i ll not draw anything and even if i did, it would not affect viewport
}
///////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////
myOpenGLRendererWindow::~myOpenGLRendererWindow()
{
    delete OGL_Context;
}
